@@ -25,6 +25,7 @@
 #include <sstream>
 #include <string>
 #include <algorithm>
+#include <cmath>
 
 #include "residfp/siddefs-fp.h"
 #include "sidplayfp/siddefs.h"
@@ -36,6 +37,8 @@
 namespace libsidplayfp
 {
 
+const char ERR_INVALID_CW[]     = "Invalid combined waveforms strength.";
+
 const char* ReSIDfp::getCredits()
 {
     return
@@ -44,14 +47,13 @@ const char* ReSIDfp::getCredits()
         "MOS6581/CSG8580 (SID) Emulation:\n"
         "\t(C) 1999-2002 Dag Lem\n"
         "\t(C) 2005-2011 Antti S. Lankila\n"
-        "\t(C) 2010-2023 Leandro Nini\n";
+        "\t(C) 2010-2024 Leandro Nini\n";
 }
 
 ReSIDfp::ReSIDfp(sidbuilder *builder) :
     sidemu(builder),
     m_sid(*(new reSIDfp::SID))
 {
-    m_buffer = new short[OUTPUTBUFFERSIZE];
     reset(0);
 }
 
@@ -66,6 +68,11 @@ void ReSIDfp::filter6581Curve(double filterCurve)
    m_sid.setFilter6581Curve(filterCurve);
 }
 
+void ReSIDfp::filter6581Range(double adjustment)
+{
+   m_sid.setFilter6581Range(adjustment);
+}
+
 void ReSIDfp::filter8580Curve(double filterCurve)
 {
    m_sid.setFilter8580Curve(filterCurve);
@@ -123,8 +130,7 @@ void ReSIDfp::sampling(float systemclock, float freq,
 
     try
     {
-        const int halfFreq = (freq > 44000) ? 20000 : 9 * freq / 20;
-        m_sid.setSamplingParameters(systemclock, sampleMethod, freq, halfFreq);
+        m_sid.setSamplingParameters(systemclock, sampleMethod, freq);
     }
     catch (reSIDfp::SIDError const &)
     {
@@ -133,6 +139,12 @@ void ReSIDfp::sampling(float systemclock, float freq,
         return;
     }
 
+    if (m_buffer)
+        delete[] m_buffer;
+
+    // 20ms buffer
+    const int buffersize = std::ceil((freq / 1000.f) * 20.f);
+    m_buffer = new short[buffersize];
     m_status = true;
 }
 
@@ -160,4 +172,29 @@ void ReSIDfp::model(SidConfig::sid_model_t model, bool digiboost)
     m_status = true;
 }
 
+// Set the emulated SID combined waveforms
+void ReSIDfp::combinedWaveforms(SidConfig::sid_cw_t cws)
+{
+    reSIDfp::CombinedWaveforms combinedWaveforms;
+    switch (cws)
+    {
+        case SidConfig::AVERAGE:
+            combinedWaveforms = reSIDfp::AVERAGE;
+            break;
+        case SidConfig::WEAK:
+            combinedWaveforms = reSIDfp::WEAK;
+            break;
+        case SidConfig::STRONG:
+            combinedWaveforms = reSIDfp::STRONG;
+            break;
+        default:
+            m_status = false;
+            m_error = ERR_INVALID_CW;
+            return;
+    }
+
+    m_sid.setCombinedWaveforms(combinedWaveforms);
+    m_status = true;
+}
+
 }
