@@ -27,22 +27,8 @@
 namespace reSIDfp
 {
 
-unsigned short Filter6581::clock(int voice1, int voice2, int voice3)
+int Filter6581::solveIntegrators()
 {
-    const int V1 = voice1;
-    const int V2 = voice2;
-    // Voice 3 is silenced by voice3off if it is not routed through the filter.
-    const int V3 = (filt3 || !voice3off) ? voice3 : 0;
-
-    int Vsum = 0;
-    int Vmix = 0;
-
-    (filt1 ? Vsum : Vmix) += V1;
-    (filt2 ? Vsum : Vmix) += V2;
-    (filt3 ? Vsum : Vmix) += V3;
-    (filtE ? Vsum : Vmix) += Ve;
-
-    Vhp = currentSummer[currentResonance[Vbp] + Vlp + Vsum];
     Vbp = hpIntegrator.solve(Vhp);
     Vlp = bpIntegrator.solve(Vbp);
 
@@ -54,9 +40,7 @@ unsigned short Filter6581::clock(int voice1, int voice2, int voice3)
     // The filter input resistors are slightly bigger than the voice ones
     // Scale the values accordingly
     constexpr int filterGain = static_cast<int>(0.93 * (1 << 12));
-    Vfilt = (Vfilt * filterGain) >> 12;
-
-    return currentVolume[currentMixer[Vmix + Vfilt]];
+    return (Vfilt * filterGain) >> 12;
 }
 
 Filter6581::~Filter6581()
